/**<pre>
 * Title: iSIM
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * @version 1.0
 * </pre>
 */

package gui;

import gui.tray.*;
import map.*;
import util.*;

import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.lang.Math;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;



/* ===================================================================== */
/** This class is an extension of a JPanel.  It's backgound is that of
 *  scale map corresponding to real world GPS coordinates.  Agents are drawn
 *  on the JPanel, and can be dragged around the map. also wirelss ranges
 *  are drawn on each Agent if requested by the iSIM user.
 *
 */
public class MapPanel extends JPanel implements MouseListener, MouseMotionListener{


	private MapFrame parent;

	/** The chart data that contains map display and gps information. */
  	private ChartData chartdata;

	/** Holds the map background image. */
	private Image img;

	/** The point of key/center agent for drawing. */
	private Point pos;

	/** Offset position on an agent when mouse-dragging. */
	private int x_offset = 0;

	/** Offset position on an agent when mouse-dragging. */
	private int y_offset = 0;

	/** A time for keeping track of double-clicking. */
	private Date doubleclick = new Date();

	/** A Dialog Window for display User information. */
	private PropDialog propDialog;

	/** A Dialog Window for display User information. */
	private RoomDialog roomDialog;

	private Building building;

	private Floor floor;

	private PnlTabWalls walls;
	private boolean wall = false;

	private PnlTabOpenings openings;
	private boolean opening = false;

	private PnlTabFurnishings furnishings;
	private boolean furnishing = false;


	private Point start;
	private Point end;

	private Object current;
	private Object clicked;


    /* ===================================================================== */
    /** The constructor sets up mouse listeners and sets up its own variables
     *  with the given ones.
     *
     *  @param  controller   The controller of simulation.
     *  @param  ranges   True if displaying wireless ranges.
     *
     */
  	public MapPanel(MapFrame p, Building b){

		parent = p;
		building = b;
		loadChart("background.xml");

		// Sign up listeners
		this.addMouseListener(this);
		this.addMouseMotionListener(this);

  	} //Constructor





    /* ===================================================================== */
    /** This method returns our preferred size which should be the size of the
     *  map image that this object displays.
     *
     *  @return  Dimension   The size of this JPanel with map image.
     *
     */
  	synchronized public Dimension getPreferredSize(){

    	if (img!=null){

			// Wait until the image is loaded so we know the size
			int i=0;
			while ((i<30) && (img.getWidth(this)<0)){

				try {this.wait(100);} catch (InterruptedException e) {;}
				i++;
			}


			return new Dimension(img.getWidth(this), img.getHeight(this));

    	}else{

      		return new Dimension(100,100);
    	}

  	} //method: getPreferredSize



    /* ===================================================================== */
    /** This method is used whenever this JPanel gets painted.  The method
     *  draws all agents onto the map, and draws their wireless ranges if
     *  wanted
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *  @param  users   The user agents to draw.
     *  @param  points   The access point agents to draw.
     *
     */
  	synchronized void draw(Graphics g){

		// Make sure there is something to draw
   		if( floor != null ){


			// Draw map information
			try{

				Vector rooms = floor.getRooms();

				for(int i=0; i<rooms.size(); i++ ){

					Room rm = (Room)rooms.elementAt(i);
					Vector walls = rm.getWalls();
					Vector furns = rm.getFurnishings();

					for(int j=0; j<walls.size(); j++ ){

						Wall wall = (Wall)walls.elementAt(j);
						Position p1 = wall.getPosition1();
						Position p2 = wall.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						if( rm.equals(current) || wall.equals(current) || floor.equals(current) )
							g.setColor(Color.black );
						else
							g.setColor(Color.lightGray );
						g.drawLine( pos1.x, pos1.y, pos2.x, pos2.y );
						g.drawLine( pos1.x+1, pos1.y, pos2.x+1, pos2.y );
						g.drawLine( pos1.x, pos1.y+1, pos2.x, pos2.y+1 );

						Vector openings = wall.getOpenings();

						for(int k=0; k<openings.size(); k++ ){

							Opening opening = (Opening)openings.elementAt(k);
							Position op1 = opening.getPosition1();
							Position op2 = opening.getPosition2();

							Point opos1 = chartdata.transPos(op1);
							Point opos2 = chartdata.transPos(op2);

							if( opening.equals(current) || rm.equals(current) || wall.equals(current) || floor.equals(current) )
								g.setColor(Color.blue );
							else{
								g.setColor(new Color(150,150,250) );
							}
							g.drawLine( opos1.x, opos1.y, opos2.x, opos2.y );
							g.drawLine( opos1.x+1, opos1.y, opos2.x+1, opos2.y );
							g.drawLine( opos1.x, opos1.y+1, opos2.x, opos2.y+1 );
							g.drawLine( opos1.x+2, opos1.y, opos2.x+2, opos2.y );
							g.drawLine( opos1.x, opos1.y+2, opos2.x, opos2.y+2 );
							g.drawLine( opos1.x-1, opos1.y, opos2.x-1, opos2.y );
							g.drawLine( opos1.x, opos1.y-1, opos2.x, opos2.y-1 );


						} //for


					} //for

					for(int j=0; j<furns.size(); j++ ){

						Furnishing furn = (Furnishing)furns.elementAt(j);
						Position p1 = furn.getPosition1();
						Position p2 = furn.getPosition2();

						Point pos1 = chartdata.transPos(p1);
						Point pos2 = chartdata.transPos(p2);

						//System.out.println("   " +pos1.x+" "+ pos1.y+" "+ (pos2.x-pos1.x)+" "+ (pos2.y-pos1.y));

						if( furn.equals(current) || rm.equals(current) || floor.equals(current) )
							g.setColor(Color.black );
						else
							g.setColor(Color.lightGray );

						int width, height, x, y;
						if( pos1.x < pos2.x ){
							x = pos1.x;
							width = pos2.x - pos1.x;
						}else{
							x = pos2.x;
							width = pos1.x - pos2.x;
						}
						if( pos1.y < pos2.y ){
							y = pos1.y;
							height = pos2.y - pos1.y;
						}else{
							y = pos2.y;
							height = pos1.y - pos2.y;
						}

						g.drawRect( x, y, width, height );
						g.drawRect( x+1, y+1, width, height );


					} //for


				} //for

			}catch(Exception ee){}

			// Draw a circle
			if( pos != null ){
				if(opening || furnishing || wall || (clicked != null) ){
					g.setColor(Color.red);
				}else{
					g.setColor(Color.lightGray);
				}
					g.drawOval(pos.x-8,pos.y-7,16,16);
					g.drawLine(pos.x-4,pos.y,pos.x-12,pos.y);
					g.drawLine(pos.x+4,pos.y,pos.x+12,pos.y);
					g.drawLine(pos.x,pos.y-4,pos.x,pos.y-12);
					g.drawLine(pos.x,pos.y+4,pos.x,pos.y+12);

			}
			pos = null;

			// Draw a circle
			if( clicked != null ){

				g.setColor(Color.green );
				Position p1;
				Position p2;
				if(clicked instanceof Wall){
					Wall wall = (Wall)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}else if(clicked instanceof Opening){
					Opening wall = (Opening)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}else{
					Furnishing wall = (Furnishing)clicked;
					p1 = wall.getPosition1();
					p2 = wall.getPosition2();
				}

				Point pos1 = chartdata.transPos(p1);
				Point pos2 = chartdata.transPos(p2);

				g.drawOval(pos1.x-8,pos1.y-7,16,16);
				g.drawLine(pos1.x-4,pos1.y,pos1.x-12,pos1.y);
				g.drawLine(pos1.x+4,pos1.y,pos1.x+12,pos1.y);
				g.drawLine(pos1.x,pos1.y-4,pos1.x,pos1.y-12);
				g.drawLine(pos1.x,pos1.y+4,pos1.x,pos1.y+12);

				g.drawOval(pos2.x-8,pos2.y-7,16,16);
				g.drawLine(pos2.x-4,pos2.y,pos2.x-12,pos2.y);
				g.drawLine(pos2.x+4,pos2.y,pos2.x+12,pos2.y);
				g.drawLine(pos2.x,pos2.y-4,pos2.x,pos2.y-12);
				g.drawLine(pos2.x,pos2.y+4,pos2.x,pos2.y+12);

			}



			// Draw current movement
			if(furnishing){

				if( start != null && end != null ){

					g.setColor(Color.red );
					int width, height, x, y;
					if( start.x < end.x ){
						x = start.x;
						width = end.x - start.x;
					}else{
						x = end.x;
						width = start.x - end.x;
					}
					if( start.y < end.y ){
						y = start.y;
						height = end.y - start.y;
					}else{
						y = end.y;
						height = start.y - end.y;
					}

					g.drawRect( x, y, width, height );
					g.drawRect( x+1, y+1, width, height );

				}

			}else if( wall ){

				if( start != null && end != null ){

					g.setColor(Color.red );
					g.drawLine( start.x, start.y, end.x, end.y );
					g.drawLine( start.x+1, start.y, end.x+1, end.y );
					g.drawLine( start.x, start.y+1, end.x, end.y+1 );

				}

			}else if( opening ){

				if( start != null && end != null ){

					g.setColor(Color.red );
					g.drawLine( start.x, start.y, end.x, end.y );
					g.drawLine( start.x+1, start.y, end.x+1, end.y );
					g.drawLine( start.x, start.y+1, end.x, end.y+1 );

				}
			}else if( clicked instanceof Opening || clicked instanceof Wall){

				if( start != null && end != null ){

					g.setColor(Color.red );
					g.drawLine( start.x, start.y, end.x, end.y );
					g.drawLine( start.x+1, start.y, end.x+1, end.y );
					g.drawLine( start.x, start.y+1, end.x, end.y+1 );

				}
			}else if( clicked instanceof Furnishing ){

				if( start != null && end != null ){

					g.setColor(Color.red );
					int width, height, x, y;
					if( start.x < end.x ){
						x = start.x;
						width = end.x - start.x;
					}else{
						x = end.x;
						width = start.x - end.x;
					}
					if( start.y < end.y ){
						y = start.y;
						height = end.y - start.y;
					}else{
						y = end.y;
						height = start.y - end.y;
					}

					g.drawRect( x, y, width, height );
					g.drawRect( x+1, y+1, width, height );

				}
			}else{
				start = null;
				end = null;
			}

		} //if


  	} //method: draw





    /* ===================================================================== */
    /** This method is used to make the JPanel re draw itself, usually because
     *  some information or location has caused an Agent to change or created
     *  a new Agent in the Simulator.
     *
     *  @param  pos   The center point, usually point of current agent.
     *  @param  users   The user agents to draw.
     *  @param  points   The access point agents to draw.
     *
     */
  	synchronized public void updatePos(Point pos){

		// Get grphics to draw
		Graphics g=getGraphics();

		// Make old drawing
		draw(g);

		// Set the point, it may be null, or real
		this.pos=pos;

		// Now redraw.
		draw(g);

		g.dispose();

  	} //method: updatePos





    /* ===================================================================== */
    /** This method is used in case of some time out event occurs to re draw
     *  this JPanel.
     *
     */
  	synchronized public void timeOut(){

    	Graphics g=getGraphics();
    	draw(g);

    	draw(g);
    	g.dispose();

  	} //method: timeOut



    /* ===================================================================== */
    /** This method is used by JPanels for updating its display, it is safe
     *  to just call paint in this JPanel.
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	public void update(Graphics g){

    	paint(g);

  	} //method: update




    /* ===================================================================== */
    /** This method will get the Graphics object for painting this JPanel, and
     *  then call on paint(Graphics g).
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	synchronized public void paint(){

    	Graphics g=getGraphics();
    	paint(g);
    	g.dispose();

  	} //method: paint



    /* ===================================================================== */
    /** This method is used to paint this JPanel, it will first make sure the
     *  map image is set, and then will call the draw() method to paint all
     *  agents and wireless ranges.
     *
     *  @param  g   The Graphics object for drawing images and circles.
     *
     */
  	synchronized public void paint(Graphics g){


		if (img!=null) {
			g.drawImage(img, 0, 0, this);
		}


		g.setPaintMode();
		g.setColor(Color.blue);

		draw(g);
		g.dispose();

	} //method: paint






    /* ===================================================================== */
    /** This method checks for two kinds of mouse clicks.  If first checks for
     *  a right mouse click which is for rotating a user.(changing user's
     *  facing direction)
     *  This mehtod also checks for mouse left button double clicks.  Based on
     *  the location of click and the timing, it will lauch a PropDialog for
     *  the User that was double clicked on.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseClicked(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();
		p = new Point( (int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10) );

		clicked = null;


		// Draw map information
		try{

			Vector rooms = floor.getRooms();

			for(int i=0; i<rooms.size(); i++ ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();
				Vector furns = rm.getFurnishings();

				for(int j=0; j<walls.size(); j++ ){

					Wall wall = (Wall)walls.elementAt(j);
					Position p1 = wall.getPosition1();
					Position p2 = wall.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					//check if a wall
					double s = 0;
					double s1 = 0;
					double s2 = 0;
					try{
						s = (pos1.getY()-pos2.getY())/(pos1.getX()-pos2.getX());
					}catch(Exception ee){}
					try{
						s1 = (pos1.getY()-p.getY())/(pos1.getX()-p.getX());
					}catch(Exception ee){}
					try{
						s2 = (p.getY()-pos2.getY())/(p.getX()-pos2.getX());
					}catch(Exception ee){}

					s = Math.abs(s);
					s1 = Math.abs(s1);
					s2 = Math.abs(s2);

					if( s > 1000000 ) s = 0;

					double d1 = s-s1;
					double d2 = s-s2;

					if( s > 2 ){
						try{
							d1 = d1/s;
						}catch(Exception ee){}
						try{
							d2 = d2/s;
						}catch(Exception ee){}
					}

					if( s1 > 1000000 || new Double(s1).isNaN() ) d1 = 0;
					if( s2 > 1000000 || new Double(s2).isNaN() ) d2 = 0;

					if( d1 < 0)
						d1 = d1 *-1.0;
					if( d2 < 0)
						d2 = d2 *-1.0;

					if( d1 < .2 && d2 < .2 && isWall(p, wall) ){
						clicked = wall;
					}


					Vector openings = wall.getOpenings();

					for(int k=0; k<openings.size(); k++ ){

						Opening opening = (Opening)openings.elementAt(k);
						Position op1 = opening.getPosition1();
						Position op2 = opening.getPosition2();

						Point opos1 = chartdata.transPos(op1);
						Point opos2 = chartdata.transPos(op2);

						//check if opening
						s = 0;
						s1 = 0;
						s2 = 0;
						try{
							s = (opos1.getY()-opos2.getY())/(opos1.getX()-opos2.getX());
						}catch(Exception ee){}
						try{
							s1 = (opos1.getY()-p.getY())/(opos1.getX()-p.getX());
						}catch(Exception ee){}
						try{
							s2 = (p.getY()-opos2.getY())/(p.getX()-opos2.getX());
						}catch(Exception ee){}

						//System.out.println(" Slope: "+s );
						//System.out.println("      Slopes: "+s1+"   "+s2 );
						//infinity check
						s = Math.abs(s);
						s1 = Math.abs(s1);
						s2 = Math.abs(s2);

						if( s > 1000000 ) s = 0;

						d1 = s-s1;
						d2 = s-s2;

						if( s > 2 ){
							try{
								d1 = d1/s;
							}catch(Exception ee){}
							try{
								d2 = d2/s;
							}catch(Exception ee){}
						}

						if( s1 > 1000000 || new Double(s1).isNaN() ) d1 = 0;
						if( s2 > 1000000 || new Double(s2).isNaN() ) d2 = 0;

						if( d1 < 0)
							d1 = d1 *-1.0;
						if( d2 < 0)
							d2 = d2 *-1.0;

						if( d1 < .2 && d2 < .2 && isOpening(p, opening) ){
							clicked = opening;
						}


					} //for


				} //for

				for(int j=0; j<furns.size(); j++ ){

					Furnishing furn = (Furnishing)furns.elementAt(j);
					Position p1 = furn.getPosition1();
					Position p2 = furn.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					if( p.x >= pos1.x && p.x <= pos2.x && p.y >= pos1.y && p.y <= pos2.y ){
						clicked = furn;
					}else if( p.x <= pos1.x && p.x >= pos2.x && p.y <= pos1.y && p.y >= pos2.y ){
						clicked = furn;
					}else if( p.x >= pos1.x && p.x <= pos2.x && p.y <= pos1.y && p.y >= pos2.y ){
						clicked = furn;
					}else if( p.x <= pos1.x && p.x >= pos2.x && p.y >= pos1.y && p.y <= pos2.y ){
						clicked = furn;
					}


				} //for

			} //for

		}catch(Exception ee){}


		if( clicked != null ){
			// Check time to see if it is double-click. double-click if < .75 sec.
			if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

				// Close old PropDialog if it exists.
				try{
					propDialog.dispose();
				}catch(Exception exc){}

				// Open a PropDialog
				propDialog = new PropDialog( clicked, parent );

			}else{
				//Get time of click for checking for double-click next time.
				doubleclick = new Date();
			}
		}else if( isRoom(p) != null ){

			// Check time to see if it is double-click. double-click if < .75 sec.
			if( ((new Date()).getTime() - doubleclick.getTime()) < 750 ){

				// Close old PropDialog if it exists.
				try{
					roomDialog.dispose();
				}catch(Exception exc){}

				// Open a PropDialog
				roomDialog = new RoomDialog( isRoom(p), parent );

			}else{
				//Get time of click for checking for double-click next time.
				doubleclick = new Date();
			}


		}

		this.repaint();


	} //method: mouseClicked





    /* ===================================================================== */
    /** This method is used to get the offset position when we click and drag
     *  an agent on the map.  Offset is needed for correctly drawing the agent
     *  on the map.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mousePressed(MouseEvent e){

		int btn = e.getButton();

		if( clicked != null ){

			Position p1;
			Position p2;
			if(clicked instanceof Wall){
				Wall wall = (Wall)clicked;
				p1 = wall.getPosition1();
				p2 = wall.getPosition2();
			}else if(clicked instanceof Opening){
				Opening wall = (Opening)clicked;
				p1 = wall.getPosition1();
				p2 = wall.getPosition2();
			}else{
				Furnishing wall = (Furnishing)clicked;
				p1 = wall.getPosition1();
				p2 = wall.getPosition2();
			}

			Point pos1 = chartdata.transPos(p1);
			Point pos2 = chartdata.transPos(p2);
			Point p = e.getPoint();
			p = new Point( (int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10) );


			if( diff(pos1,p) < 10 ){
				//System.out.println("      "+diff(pos1,p)+"    "+diff(pos2,p));
				start = pos2;
				end = pos1;
				pos = end;

			}else if( diff(pos2,p) < 10 ){
				//System.out.println("     "+diff(pos1,p)+"    "+diff(pos2,p));
				start = pos1;
				end = pos2;
				pos = end;
			}else{

				clicked = null;
			}

			repaint();


		}else{
			// See if we are clicking and dragging with left button.
			if( btn == 1 ){

				Point p = e.getPoint();
				start = new Point( (int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10) );

			}
		}

	} //method: mousePressed




    /* ===================================================================== */
    /** This method lets the object know that the user has released from a
     *  mouse drag.  It sends a location change event, so that the location
     *  of the current user can be sent to the iPAQ application.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseReleased(MouseEvent e){

		int btn = e.getButton();
		Point p = e.getPoint();

		if(clicked != null ){

			if(clicked instanceof Wall){
				Wall wall = (Wall)clicked;
				wall.setPosition1(chartdata.transPos(start));
				wall.setPosition2(chartdata.transPos(end));
			}else if(clicked instanceof Opening){
				Opening wall = (Opening)clicked;
				wall.setPosition1(chartdata.transPos(start));
				wall.setPosition2(chartdata.transPos(end));
			}else{
				Furnishing wall = (Furnishing)clicked;
				wall.setPosition1(chartdata.transPos(start));
				wall.setPosition2(chartdata.transPos(end));
			}
			clicked = null;

		}else{

		if( btn == 1 ){
			//do some stuff then...

			if(wall){

				walls.addWall( chartdata.transPos(start), chartdata.transPos(end) );

			}else if(furnishing){

				furnishings.addFurnishing( chartdata.transPos(start), chartdata.transPos(end) );

			}else if(opening){

				// This junk makes sure an opening is on the current wall.
				if( current instanceof Wall ){

					Position p1 = ((Wall)current).getPosition1();
					Position p2 = ((Wall)current).getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					double s = 0;
					double s1 = 0;
					double s2 = 0;
					try{
						s = (pos1.getY()-pos2.getY())/(pos1.getX()-pos2.getX());
					}catch(Exception ee){}
					try{
						s1 = (pos1.getY()-start.getY())/(pos1.getX()-start.getX());
					}catch(Exception ee){}
					try{
						s2 = (pos1.getY()-end.getY())/(pos1.getX()-end.getX());
					}catch(Exception ee){}

					//System.out.println(" Slope: "+s );
					//System.out.println("      Slopes: "+s1+"   "+s2 );

					//infinity check
					s = Math.abs(s);
					s1 = Math.abs(s1);
					s2 = Math.abs(s2);

					if( s > 1000000 ) s = 0;

					double d1 = s-s1;
					double d2 = s-s2;

					if( s > 2 ){
						try{
							d1 = d1/s;
						}catch(Exception ee){}
						try{
							d2 = d2/s;
						}catch(Exception ee){}
					}

					if( s1 > 1000000 || new Double(s1).isNaN() ) d1 = 0;
					if( s2 > 1000000 || new Double(s2).isNaN() ) d2 = 0;

					if( d1 < 0)
						d1 = d1 *-1.0;
					if( d2 < 0)
						d2 = d2 *-1.0;

					if( d1 < .2 && d2 < .2 ){
						openings.addOpening( chartdata.transPos(start), chartdata.transPos(end) );
					}else{
						start = null;
						end = null;
					}

				}
			}
		}
		}


	} //method: mouseReleased




    /* ===================================================================== */
    /** This method is for moving an Agent around on the map.  It determines
     *  the closest Agent to the drag point, determines if that point is on
     *  an Agent, and then redraws the Agent to the new location based on
     *  offsets gotten from mousePressed method.
     *
     *  @param  e   The incoming MouseEvent containing button and point info.
     *
     */
	public void mouseDragged(MouseEvent e){

		//System.out.println("drag");

		Point p = e.getPoint();
		int btn = e.getButton();

		if( btn == 0 ){

			end = new Point( (int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10) );
			pos = end;

			repaint();

		} //if( btn == 0 )

	} //method: mouseDragged



    /* ===================================================================== */
    /** This method
     *
     *  @param  e
     *
     */
 	public void mouseMoved(MouseEvent e){

		Point p = e.getPoint();
		p = new Point((int)(Math.round((p.getX()-2)/10)*10), (int)(Math.round((p.getY()-2)/10)*10));
		pos = p;

		Position chart_pos = chartdata.transPos(pos);

		parent.setCoordinate("Lat: "+ (int)(chart_pos.getLatitude()/100) +"."+ (int)(((chart_pos.getLatitude()/100)-(int)(chart_pos.getLatitude()/100))/60*100000000) + ",   Lng: "+(int)(chart_pos.getLongitude()/100) +"."+ (int)(((chart_pos.getLongitude()/100)-(int)(chart_pos.getLongitude()/100))/60*-100000000) );

		repaint();

	} //method: mouseMoved



    /* ===================================================================== */
    /** This method not used.
     *
     *  @param  e
     *
     */
	public void mouseEntered(MouseEvent e){} //method: mouseEntered
    /* ===================================================================== */
    /** This method not used.
     *
     *  @param  e
     *
     */
	public void mouseExited(MouseEvent e){} //method: mouseExited








	public Building getBuilding(){

		return building;

	}

	public void setBuilding(Building b){

		building = b;

	}

	public MapFrame getMapFrame(){

		return parent;

	}


	public void setFloor(Floor f){

		floor = f;
	}



	public void done(){

		wall = false;
		opening = false;
		furnishing = false;

		start = null;
		end = null;

	}


	public void addWall(PnlTabWalls walls){

		wall = true;
		this.walls = walls;
	}


	public void addFurnishing(PnlTabFurnishings furnishings){

		furnishing = true;
		this.furnishings = furnishings;
	}


	public void addOpening(PnlTabOpenings openings){

		opening = true;
		this.openings = openings;
	}


	public void setCurrent(Object o){
		current = o;
	}




	//If the given point is in the room then it must
	// have a wall on each side.
	public Room isRoom(Point p){

		if( floor != null ){

			Vector rooms = floor.getRooms();

			for(int i=0; i<rooms.size(); i++ ){

				Room rm = (Room)rooms.elementAt(i);
				Vector walls = rm.getWalls();

				boolean north = false;
				boolean south = false;
				boolean east = false;
				boolean west = false;

				for(int j=0; j<walls.size(); j++ ){

					Wall wall = (Wall)walls.elementAt(j);
					Position p1 = wall.getPosition1();
					Position p2 = wall.getPosition2();

					Point pos1 = chartdata.transPos(p1);
					Point pos2 = chartdata.transPos(p2);

					//north
					if( pos1.y > p.y && pos2.y > p.y ){
						if( pos1.x < p.x && pos2.x > p.x )
							north = true;
						if( pos1.x > p.x && pos2.x < p.x )
							north = true;
					//south
					}else if( pos1.y < p.y && pos2.y < p.y ){
						if( pos1.x < p.x && pos2.x > p.x )
							south = true;
						if( pos1.x > p.x && pos2.x < p.x )
							south = true;
					//west
					}else if( pos1.x > p.x && pos2.x > p.x ){
						if( pos1.y < p.y && pos2.y > p.y )
							east = true;
						if( pos1.y > p.y && pos2.y < p.y )
							east = true;
					//west
					}else if( pos1.x < p.x && pos2.x < p.x ){
						if( pos1.y < p.y && pos2.y > p.y )
							west = true;
						if( pos1.y > p.y && pos2.y < p.y )
							west = true;
					}
				}

				if( north && south && east && west ){
					return rm;
				}

			} //for



		}

		return null;
	}


	//If the given point is in the room then it must
	// have a wall on each side.
	public boolean isWall(Point p, Wall wall){

		boolean x = false;
		boolean y = false;


		Position p1 = wall.getPosition1();
		Position p2 = wall.getPosition2();

		Point pos1 = chartdata.transPos(p1);
		Point pos2 = chartdata.transPos(p2);

		//
		if( pos1.y >= p.y && pos2.y <= p.y ){
			x = true;
			//System.out.println("xxxxxxxxxxxxxxxxxxxxxxxxxx");
		//
		}else if( pos1.y <= p.y && pos2.y >= p.y ){
			x = true;
			//System.out.println("xxxxxxxxxxxxxxxx");
		//
		}
		if( pos1.x >= p.x && pos2.x <= p.x ){
			y = true;
			//System.out.println("yyyyyyyyyyyyyyyyyyyyyyyyyy");
		//
		}else if( pos1.x <= p.x && pos2.x >= p.x ){
			y = true;
			//System.out.println("yyyyyyyyyyyyyyyyyy");
		}


		if( x && y ){
			return true;
		}else{
			return false;
		}
	}

	//If the given point is in the room then it must
	// have a wall on each side.
	public boolean isOpening(Point p, Opening open){

		boolean x = false;
		boolean y = false;

		Position p1 = open.getPosition1();
		Position p2 = open.getPosition2();

		Point pos1 = chartdata.transPos(p1);
		Point pos2 = chartdata.transPos(p2);

		//
		if( pos1.y >= p.y && pos2.y <= p.y ){
			x = true;
		//
		}else if( pos1.y <= p.y && pos2.y >= p.y ){
			x = true;
		//
		}

		if( pos1.x >= p.x && pos2.x <= p.x ){
			y = true;
		//
		}else if( pos1.x <= p.x && pos2.x >= p.x ){
			y = true;
		}


		if( x && y ){
			return true;
		}else{
			return false;
		}
	}



    /* ===================================================================== */
    /** This method returns the pixel distance between two points.
     *
     *  @param  p1   The first point.
     *  @param  p2   The second point.
     *
     */
	private double diff(Point p1, Point p2){

		double x = p1.getX() - p2.getX();
		if( x < 0 )
			x = x * -1;

		double y = p1.getY() - p2.getY();
		if( y < 0 )
			y = y * -1;


		return Math.sqrt( x*x + y*y );

	} //method: diff




    /* ===================================================================== */
    /** This method returns the chartdata kept inside this object used for
     *  turning pixel coordinates into real GPS coordinates.
     *
     *  @return  ChartData   The chart data used in this object to return.
     *
     */
	public ChartData getChartData(){

		return chartdata;

	} //method: getChartData




    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.  Then the new
     *  ChartData will give us the map image to use.
     *
     *  @param  name   file name for new data to use.
     *
     */
	public void loadChart(String name){

		this.setVisible(false);

		this.readData(name);

		if (img!=null) {img.flush();}

		img = getToolkit().getImage("files/data/"+chartdata.imgFileName);


		this.setSize(this.getPreferredSize());


		this.setVisible(true);
		repaint();

	} //method: loadChart



    /* ===================================================================== */
    /** This method loads new chart data, by first reading the given xml file
     *  which contains map name and reference point information.
     *
     *  @param  file_name   Name of XML file that contains new ChartData info.
     *
     */
 	public void readData(String file_name){

    	chartdata = null;

		InputParser ip = new InputParser(file_name);

		try{
			chartdata=ChartData.loadChartData(ip);
		}catch (IOException e) {}


  	} //method: readData


} //class: ChartComp

