/**<pre>
 * Title: GPS Simulator
 * Copyright: Copyright (c) 2002
 * Company:   University of Oregon Computer Science Dept.
 *
 * @author Jason Prideaux
 * adapted from: Henrik Bjorkman
 * @version 1.0
 * </pre>
 */

package util;

import java.io.*;
import java.awt.*;
import java.text.*;


/* ====================================================================== */
/** This class represents a reference point on a map image.  A reference
 *  point is where pixel x equals a longitude value, and pixel y equals a
 *  latitude value.  Thus every ReferencePoint object must contain an x pixel,
 *  a y pixel, and the corresponding longitude and latitude value.
 *
 *  This is class is needed so that we can take a pixel point on a map and
 *  determine its latitude/longitude in the real world.  And thus we can send
 *  real latitude longitude values to the ipaq gps receiver program.
 *
 */
public class ReferencePoint implements java.io.Serializable{

	/** in pixels */
	public int x,y;

	/** in minutes */
	public double lat,lng;

	/** Used to convert latitude/longitude String to double. */
	public static final double MAX_NORTH=90*60;

	/** Used to convert latitude/longitude String to double. */
	public static final double MAX_SOUTH=-90*60;

	/** Used to convert latitude/longitude String to double. */
	public static final double MAX_WEST=-180*60;

	/** Used to convert latitude/longitude String to double. */
	public static final double MAX_EAST=180*60;



    /* ===================================================================== */
    /** This Constructor creates a reference Point object with the x and y
     *  pixel location and the corresponding latidude and longitude
     *
     *  @param  xr   The x pixel.
     *  @param  yr   The y pixel.
     *  @param  latr   The latitude.
     *  @param  lngr   The longitude.
     *
     */
  	public ReferencePoint(String xr, String yr, String latr, String lngr){

		this.x=Integer.parseInt(xr);
		this.y=Integer.parseInt(yr);
		setLat(latr);
		setLng(lngr);

  	} //Constructor




 	public void setLat(String str){

    	lat=parseLatLng(str,'N','S');
  	} //method:






  	public void setLng(String str){

    	lng=parseLatLng(str,'E','W');
  	} //method:





  	public static double parseLatLng(String str, char pos, char neg) throws NumberFormatException{

		int l=numLen(str);

		if ((l==0) || (l==str.length())){

		  	throw new NumberFormatException("No lat/lng found in: "+str);
		}

		double hr=Double.valueOf(str.substring(0,l)).doubleValue();
		double min=Double.valueOf(str.substring(l+1)).doubleValue();

		if (hr>180.1){

		  	throw new NumberFormatException("Out of range lat/lng: "+str);
		}

		if (min>60.1){

		  	throw new NumberFormatException("Bad minutes in lat/lng: "+str);
		}

		double m=hr*60d+min;

		if (str.charAt(l)==neg){

		  m=-m;

		}else if (str.charAt(l)!=pos){

			throw new NumberFormatException("Wrong format lat/lng: "+str);
		}

		return(m);


  	} //method:





	public static int numLen(String str){

		int i=0;
		while (i<str.length() && Character.isDigit(str.charAt(i))) {i++;}
		return(i);

	} //method:





	public static String toStringLatLng(double minutes, char pos, char neg){

		char sign=pos;
		int l=(new Double(minutes*1000)).intValue();
		if( l<0 ){
			l=-l;sign=neg;
		}

		int d=l/60000;
		int m=(l%60000)/1000;
		int m1000=l%1000;
		return(ChartWord.addZero(""+d,2)+sign+
			 ChartWord.addZero(""+m,2)+"."+
			 ChartWord.addZero(""+m1000,3));

	} //method:





  	public String toString(){

    	return(""+x+" "+y+" "+toStringLatLng(lat,'N','S')+" "+toStringLatLng(lng,'E','W'));

  	} //method: toString



} //class: ReferencePoint

